@@ -3,9 +3,9 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 from mmcv.cnn import ConvModule, kaiming_init, normal_init, xavier_init
+from mmcv.runner import force_fp32
 
-from mmdet.core import (build_bbox_coder, force_fp32, multi_apply,
-                        multiclass_nms)
+from mmdet.core import build_bbox_coder, multi_apply, multiclass_nms
 from mmdet.models.builder import HEADS, build_loss
 from mmdet.models.losses import accuracy
 
@@ -244,13 +244,13 @@ class SABLHead(nn.Module):
         reg_fy = torch.transpose(reg_fy, 1, 2)
         return reg_fx.contiguous(), reg_fy.contiguous()
 
-    def reg_pred(self, x, offfset_fcs, cls_fcs):
-        """Predict bucketing esimation (cls_pred) and fine regression (offset
+    def reg_pred(self, x, offset_fcs, cls_fcs):
+        """Predict bucketing estimation (cls_pred) and fine regression (offset
         pred) with side-aware features."""
         x_offset = x.view(-1, self.reg_in_channels)
         x_cls = x.view(-1, self.reg_in_channels)
 
-        for fc in offfset_fcs:
+        for fc in offset_fcs:
             x_offset = self.relu(fc(x_offset))
         for fc in cls_fcs:
             x_cls = self.relu(fc(x_cls))
@@ -274,6 +274,15 @@ class SABLHead(nn.Module):
         feat = torch.cat([feat_fl, feat_fr], dim=-1)
         return feat
 
+    def bbox_pred_split(self, bbox_pred, num_proposals_per_img):
+        """Split batch bbox prediction back to each image."""
+        bucket_cls_preds, bucket_offset_preds = bbox_pred
+        bucket_cls_preds = bucket_cls_preds.split(num_proposals_per_img, 0)
+        bucket_offset_preds = bucket_offset_preds.split(
+            num_proposals_per_img, 0)
+        bbox_pred = tuple(zip(bucket_cls_preds, bucket_offset_preds))
+        return bbox_pred
+
     def reg_forward(self, reg_x):
         outs = self.side_aware_feature_extractor(reg_x)
         edge_offset_preds = []
